function Application(initCallback, options) {
	
	var opt = options || {}
	this.optAutoHeader = typeof (opt['autoHeader']) == 'undefined' ? true : opt['autoHeader'];
	this.optAutoHeaderText = typeof (opt['autoHeaderText']) == 'undefined' ? true : opt['autoHeaderText'];
	
	this.nav = []

	// If no screen selected - redirect to default screen
	if (!this.screen())
		this.setScreen(Application.defaultScreen);

	var that = this
	var slideFunc = function(event) {
		return that.slideClick(event)
	}
	
	that.addScreenHeader().setScreen()

	$(Application.slideSelector.concat(Application.backSelector).join(','))
		.click(slideFunc).live('click', slideFunc)
        
	// Add the hardware back-button handler
	document.addEventListener("backbutton", function() { that.back() }, false)
        
	$.isFunction(initCallback) && initCallback.call(that)
}
Application.defaultScreen = 'home';
Application.slideSelector = ['body > div.screen a[href^=#id=]'];
Application.backSelector = ['body > div.screen a[href^=#back]'];

Application.fireEvent = function(type, data) {
	var e = document.createEvent("Events")
	e.initEvent(type, false, false)
	e.data = data || null
	document.dispatchEvent(e)
} 

Application.prototype.screen = function() {
    return Hash.get('id')
}

/**
 * from - jQuery object
 * to - jQuery object
 * isReverse - boolean
*/
Application.prototype.slide = function(from, to, isReverse, callback) {
	var
		dir = isReverse ? ' reverse' : '',
		that = this;
	
	to.addClass('slide in active' + dir)
	from.addClass('slide out' + dir).one('webkitAnimationEnd', function() {
		window.top.scrollTo(0, 1)

		from.removeClass('active slide out' + dir)
		to.removeClass('slide in' + dir)
		
		Hash.go(Hash.set("id=" + to.attr('id') ))
		
		to.attr("title") && (document.title = to.attr("title"))
		$.isFunction(callback) && callback.call(this)
	})

	return false
}

Application.prototype.setScreen = function(screenId) {
	var id = screenId || this.screen()
	$('body > .screen.active').removeClass('active')
	var scr = $('#'+id+'.screen')
	scr.addClass('active')
	scr.attr("title") && (document.title = scr.attr("title"))
	this.navPush(id)
	Hash.go(Hash.set('id='+id))
	return this
}

Application.prototype.navPush = function(screen) {
	if ( this.navTop() != screen)
		this.nav.push(screen)
	if (this.nav.length == 0)
		this.nav.push(Application.defaultScreen)
}

Application.prototype.navPop = function() {
	this.nav.pop()
	return this.nav.length == 0 ? Application.defaultScreen : this.nav[this.nav.length-1]
}

Application.prototype.navEmpty = function() { return this.nav.length <= 1 }

Application.prototype.navContains = function(screenId) { return this.nav.indexOf(screenId)>-1 }

Application.prototype.navTop = function() { return this.nav[this.nav.length-1] }

Application.prototype.back = function(a) {
	var gotoHomeBeforeExit = (this.navEmpty() && (this.screen() != Application.defaultScreen))
	
	if (this.navEmpty() && !gotoHomeBeforeExit) {
		navigator.app && navigator.app.exitApp()
		return false
	}
	
	var to = this.navPop()
	a && $(a).addClass('running')
	this.slide( $('#'+this.screen()), $('#'+to), true , function() {
		a && $(a).removeClass('running')
	})
	return false
}

Application.prototype.slideClick = function(event) {
	var a = $(event.target)
	if ( a.attr('href')=='#back' )
		return this.back(a)

	var fromId = this.screen(),
		toId = $.toQueryParams(a.attr('href'))['id'] //|| a.attr('href').substring(1) || ''

	if (fromId == toId || $('#'+toId).size()<=0) {
		console.warn("Unknown or the same screen to slide: #" + toId)
		return false
	}

	this.navPush(toId)
	//a.addClass('running')
	return this.slide($('#'+fromId), $('#'+toId), !this.navContains(toId)/*, function() {a.removeClass('running')}*/ )
	
}

Application.prototype.addScreenHeader = function(screenSelector) {
	var optAutoHeaderText = this.optAutoHeaderText;
	this.optAutoHeader && $(screenSelector || "body > div.screen[title]").each(function(i, it){
		if (!$("> .header", it).size())
			$('<div class="header"><h1 data-lang="">'+(optAutoHeaderText ? $(it).attr("title") : '')+'</h1></div>').prependTo($(it))
	})
	return this // we love calls chaining
}
