var LANG = 'en', 	// It will be detected from PhoneGap later
	LangData2 = {} 	// Will be loaded later via NativeResources Phonegap Plugin 

// console stub
window.console = window.console || {
	log: function() {},
	warn: function() {}
}

function lang(it) {
	return LangData2[it] || it
}

function applyLangugeData() {
	$('*[data-lang], body > div.screen[title]').each(function(i, el){
		var it = $(el)
		if (it.attr('title'))
			it.attr('title', lang(it.attr('title')))
		if (it.attr('data-lang'))
			it.html( lang(it.attr('data-lang')) )
	})
}

var dataSource = new ContentDataSource({
	onLoadStart: function(src) {
		navigator.notification.progressStart( lang('loading')+'...', lang('please_wait'), {
			cancelable: false,
			style:  Notification[ src=='getContent' ? 'PROGRESS_STYLE_HORIZONTAL' : 'PROGRESS_STYLE_SPINNER' ]
		} )
	},
	onLoadEnd: function(src) {
		navigator.notification.progressStop()
	},
	onProgress: function(current, total) {
		navigator.notification.progressValue( current, total )
	},
	onError: function(message, src) {
		message = lang(message) //translate message
		
		if (typeof arguments.callee.failCounter == 'undefined') // Fails counter. 'Static' function-wide variable
			arguments.callee.failCounter = 0;
		
		console.error("[" + src + "] " + message)
		navigator.notification.progressStop()
		
		if (src == 'getContent') {
			var failCounter = ++arguments.callee.failCounter;
			navigator.notification.alert(message, 
				function(){ 
					if (failCounter < 3) 
						dataSource.getContent()
					else { 
						navigator.notification.progressStop()
						navigator.notification.toast(message)
						setTimeout(function(){ navigator.app && navigator.app.exitApp() }, 3000);
					}
						
				}, 
				lang("update_error"), 
				lang("try_again")
			)
			return
		}
		navigator.notification.toast(message)
	},
	onContentReceived: function(data) { //this -> ContentDataSource
		PhoneGap.exec(null, null, 'App', 'showWebView', [])
		console.log('onContentReceived')
		
		var _dataSource = this;
		
		// Full 'free' pseudo category randomly from 'paid'
		delete data.content['free'] // remove obsolete `free` content category
		
		// Fill `free` category with random content from `paid` category
		var cnt = data.content['paid'].length;
		var free = []
		while (free.length < Math.min(cnt,3) ) {
			var it = data.content.paid[Math.floor(Math.random() * cnt)]
			var found = -1
			for (var i=0,l=free.length; i<l; i++) {
				if (free[i].id != it.id) 
					continue;
				found = i;
			}
			if (found<0)
				free.push(it)
		}
		data.content.free = free;
		
		// put data received to DOM
		$.each(['free', 'paid'], function(_t, t) {
			
			if (data.content[t].length > 0)
				$('#'+t+'content').empty()
			
			$.each(data.content[t], function(i, it){
				$(
				'<li><a href="'+it.id+'" class="video" rel="'+it.id+'">' +
					'<img src="'+it.imageData+'" alt="" />' +
					'<div>'+(it.name || '')+'</div>' + 
					'<div class="play">'+lang('Play')+' <small>&#9654;</small></div>' + 
					'<div style="text-align:right;">'+(it.duration || '')+'</div>' + 
				'</a></li>'
				).appendTo('#'+t+'content')
			})
				
		})// free | paid
	}
});


document.addEventListener('optionsItemSelected', function(ev) {
	if (ev.data && (ev.data = 'menu_update')) {
		LocalStorage.clear()
		dataSource.getContent()
	}
}, false);


document.addEventListener('deviceready', function() {
	
	// Unique device ID
	DEVICE = window.device 
		? window.device.name + '/' + window.device.version + '@' + window.device.uuid
		: '-' ;
	
	// Detect language and change global variables
	LANG = (window.plugins.resources ? window.plugins.resources.getCurrentLanguage() : Hash.get('lang')) || 'en'
	LangData2 = window.plugins.resources ? window.plugins.resources.getStrings() : {};
	
	// Add some informational data to support email
	if (window.device) {
		var href = $('#support_email').attr('href')
		href += '?subject='+encodeURIComponent(lang('app_name')+" - "+window.device.name+"/"+window.device.version)
		$('#support_email').attr('href', href)
	}
	
	dataSource.setLanguage(LANG)
	applyLangugeData()
	
	new Application(function() { //this = Application itself
		dataSource.getContent()
		$('ul.videos-list li a.video').live('click', playVideoClick)
	}, {
		autoHeader: true, 
		autoHeaderText: false
	})	
	
}, false )


$(function() {
	// This is a stub for Desktop Safari5
	if (navigator.userAgent.match(/Version\/5[\.\d ]+ Safari/)) {
		// create stub for notification.*
		if (typeof(navigator.notification) == 'undefined') {
			navigator.notification = {
				progressStart: $.noop,
				progressStop: $.noop,
				progressValue: $.noop,
				alert: function(s) {alert(s)},
				toast: function(s) {alert(s)}
			}
		}
		window.plugins.innerActive = {
			play: function (url) {
				location.href = url
			}
		}
		
		window.old_prompt = window.prompt;
		window.prompt = function(title, defValue) {
			if (defValue.substring(0,4)==='gap:')
				return {status: 1, message: 'OK'}
			return window.old_prompt.apply(null, title, defValue)
		}
		
			
		// Fire onDeviceReady event manually
		setTimeout(function(){
			if (!PhoneGap.onDeviceReady.fired)  
				PhoneGap.onDeviceReady.fire()
		}, 1000)
	}

})

function playVideoClick(ev) {
	dataSource.getWatchUrl($(this).attr("rel"), function(resp) {
		resp.url && window.plugins.innerActive.play(resp.url)
	})
	return false;
}


function debug(obj) {
	var s = '';
	for (var i in obj) 
		s += i + ' = ' + obj[i] + "\n"
	console.log("debug() <<<--- \n" + s + "\ndebug() --->>>")
}

